@@ -173,7 +173,6 @@ class DApi2(object):
             regs = list(map(itemgetter(1), g))
             self.writeRegs(regs)
             
-        
 
 class DApiAccessLevel(DIntEnum):
     '''Enumeration for DAPI2 access level'''
@@ -399,6 +398,10 @@ class DApi2Commands(MetaDApi2Commands):
         msg = dmsg.CommandMessage(DApi2Commands.DISCONNECT)
         msg = self._sendCmd(msg)
         
+    def reboot(self):
+        msg = dmsg.CommandMessage(DApi2Commands.REBOOT)
+        msg = self._sendCmd(msg)
+        
     def standby(self):
         msg = dmsg.CommandMessage(DApi2Commands.STANDBY)
         msg = self._sendCmd(msg)
