@@ -11,13 +11,13 @@ from enum import IntEnum
 from dapi2.dreg.register import Register, RegBit, DRegUndefinedError
 from dapi2.dreg.group import RegGroup
 from dapi2.dapi2 import DApiAccessLevel, DApiAdditionalBoard
-from dapi2.common import versionToWord, dateToWord, wordToDate, wordToVersion
+from dapi2.common import versionToWord, dateToWord, wordToDate, wordToVersion, DAPI_ACCESS_LEVEL_MASK
 from dapi2.dmsg.message import CommandMessage
+from dapi2.signal import DSignal
 
 from .common import DBoardPreferredDapiMode, LastReset, ValueRange, DBoardException, SystemModeConfig
 from .workspace import WorkspacesContainer, StandbyWorkspace, Workspace, BaseWorkspace
-from .analoginput import PressureSensor, ElectricalSensor
-from .analoginput import PowerSupply, ReferenceSensor
+from .analoginput import PressureSensor, Analog0Sensor, Analog1Sensor, PowerSupply, ReferenceSensor
 from .debug import DebugValue
 
 
@@ -29,10 +29,24 @@ class DacChannel(IntEnum):
 class DacSignal(IntEnum):
     pass
 
+
+class WorkspaceChangedSignal(DSignal):
+    
+    def emit(self, workspace):
+        for callback in self.callbacks:
+            callback(workspace)    
+
+
+class ConnectionChangedSignal(DSignal):
+    
+    def emit(self, state, level):
+        for callback in self.callbacks:
+            callback(state, level)    
+
 class BaseDBoard(object):
     '''Base class for Dassym electronic boards.
     
-    :param DApi2 dapi: The DAPI2 object.
+    :param DApi2 _dapi: The DAPI2 object.
     :param DBoardPreferredDapiMode dmode: Specifies the preferred DAPI2 operating mode (default: `REGISTER`)
     
     '''
@@ -46,6 +60,9 @@ class BaseDBoard(object):
     wait_after_reprogramming = 8.0 #[s]
     '''Timeout after programming to reconnect to the board expressed in second.'''
     
+    wait_after_reboot = 2.0 #[s]
+    '''Timeout after reboot to reconnect to the board expressed in second.'''
+    
     @classmethod
     def getBoardClasses(cls):
         '''Returns all known board classes
@@ -90,7 +107,7 @@ class BaseDBoard(object):
         '''Constructor'''
         self.log = logging.getLogger(self.__class__.__name__)
         self.log.debug('Construct')
-        self.dapi = dapi
+        self._dapi = dapi
         self.dmode = dmode
         self._workspace = None
         self._workspaces = WorkspacesContainer(self)
@@ -100,6 +117,10 @@ class BaseDBoard(object):
         self.powersupply_range = ValueRange(30,36)
         self.analog_input_range = (ValueRange(0,10000),ValueRange(0,10000))
         
+        self._workspace_changed = WorkspaceChangedSignal()
+        self._connection_changed = ConnectionChangedSignal()
+        self._dapi.dcom.connectionChanged.connect(self.onDComConnectionChanged)
+        
     def __str__(self):
         return self.getName()
         
@@ -108,9 +129,10 @@ class BaseDBoard(object):
         
         The connection with the hardware must be established.
         '''
-        self.log.debug('initialize')
-        self.getRegisters('header')
-        self.dapi.readRegs(self.regs.pcr)
+        self.log.info('Board initialization ...')
+        self.log.debug(' - load all undefined registers')
+        
+        self.refreshAll(undefined_only=True)
         self._workspaces.clear()
         self._workspace = StandbyWorkspace(self)
         self._workspaces.append(self._workspace)
@@ -119,20 +141,26 @@ class BaseDBoard(object):
             ws_par = i+1
             ws_pcr = self.regs.pcr[i]
             if ws_pcr.value != 0:
-                self.log.debug('init workspace #{}'.format(ws_par))
+                self.log.debug(' - init workspace #{}'.format(ws_par))
                 ws = Workspace(self, 'Workspace #'+str(ws_par) , ws_par, ws_pcr)
                 self._workspaces.append(ws)
                 if self.regs.par.value == ws_par:
-                    self.log.debug('#{} is active'.format(ws_par))
+                    self.log.debug('- init : Workspace #{} is active'.format(ws_par))
                     self._workspace = ws
         if len(self._workspaces) == 1:
-            if self.dapi.dev_mode:
+            if self._dapi.dev_mode:
                 self.log.warning('No workspace found!')
             else:
                 raise DBoardException(self,'initialize', 'No workspace found!')  
             
+        
+        self._connection_changed.emit(self.dcom.isOpen(), self.getAccessLevel())
+        self._workspace_changed.emit(self._workspace)
+
         self.regs.par.changed.connect(self.onPARChanged)
+        self.regs.scsr.changed.connect(self.onSCSRChanged)
             
+        self.log.debug('Board initialization is done.')
                     
         
     def setReg(self, reg, value, synchronous=True):
@@ -149,11 +177,11 @@ class BaseDBoard(object):
         :param bool synchronous: Specifies whether the modification is synchronous (default) or not.
         '''
         if not isinstance(reg, Register):
-            reg = self.dapi.regs.getRegister(reg)
-        reg.alter(value)
+            reg = self._dapi.regs.getRegister(reg)
+        reg.set(value)
         if synchronous:
-            self.dapi.syncRegs()
-            #self.dapi.writeReg(reg)
+            self._dapi.syncRegs()
+            #self._dapi.writeReg(reg)
    
             
     def setBit(self, bit, value=1, synchronous=True):
@@ -168,13 +196,12 @@ class BaseDBoard(object):
         :param bool synchronous: Specifies whether the modification is synchronous (default) or not.
         '''
         if not isinstance(bit, RegBit):
-            bit = self.dapi.getBit(bit)
-        if bit.parent.isUndefined() and not self.dapi.dev_mode:
-            self.dapi.readReg(bit.parent)
-        v = bit.set(value) 
-        bit.parent.alter( v )
+            bit = self._dapi.getBit(bit)
+        if bit.parent.isUndefined() and not self._dapi.dev_mode:
+            self._dapi.readReg(bit.parent)
+        bit.set(value) 
         if synchronous:
-            self.dapi.syncRegs()
+            self._dapi.syncRegs()
             
     def clearBit(self, bit, synchronous=True):
         '''Clear the registers's bit.
@@ -199,10 +226,10 @@ class BaseDBoard(object):
         :param bool synchronous: Specifies whether the modification is synchronous (default) or not.
         '''
         if not isinstance(bit, RegBit):
-            bit = self.dapi.getBit(bit)
+            bit = self._dapi.getBit(bit)
         bit.parent.alter( bit.toggle() )
         if synchronous:
-            self.dapi.syncRegs()
+            self._dapi.syncRegs()
 
     def setRegs(self, regs, values, synchronous=True):
         '''Sets values to a list of registers.
@@ -214,11 +241,11 @@ class BaseDBoard(object):
         '''
         for i, reg in enumerate(regs):
             if not isinstance(reg, Register):
-                reg = self.dapi.regs.getRegister(reg)
+                reg = self._dapi.regs.getRegister(reg)
             reg.alter(values[i])
         if not synchronous:
-            self.dapi.syncRegs()
-            #self.dapi.writeRegs(regs)
+            self._dapi.syncRegs()
+            #self._dapi.writeRegs(regs)
             
     def setRegisters(self, synchronous=False, **regvalues):
         '''Sets values to a list of registers.
@@ -228,32 +255,37 @@ class BaseDBoard(object):
         '''
         for reg, value in regvalues.items():
             if not isinstance(reg, Register):
-                reg = self.dapi.regs.getRegister(reg)
+                reg = self._dapi.regs.getRegister(reg)
             self.setReg(reg, value, synchronous=False)
         if synchronous:
-            self.dapi.syncRegs()
+            self._dapi.syncRegs()
 
-    def getRegisters(self, *registers, refresh=False):
+    def getRegisters(self, *registers, refresh=False, undefined_only=False):
         '''Gets values of a list of registers.
-        
-        :param regvalues: Several arguments named according to the name of the registers to be modified.
-        :param bool refresh: True, force read registers form board, else read only undefined registers.
+        Args:
+            regvalues: Several arguments named according to the name of the registers to be modified.
+            refresh (bool): True, force read registers form board, else read only undefined registers.
+            undefined_only (bool): If True, refresh on only the undefined registers 
         '''
         regs = []
         for reg in registers:
             if not isinstance(reg, Register):
-                reg = self.dapi.regs(reg)
+                reg = self._dapi.regs(reg)
             
             if isinstance(reg, RegGroup):
                 regs.extend(reg.regs) 
             else:
                 regs.append(reg)
         if refresh:
-            self.dapi.readRegs(*regs)
+            if undefined_only:
+                uregs = [r for r in regs if r.isUndefined()]
+                self._dapi.readRegs(*uregs)
+            else: 
+                self._dapi.readRegs(*regs)
         else:
             uregs = [r for r in regs if r.isUndefined()]
-            self.dapi.readRegs(*uregs)
-        
+            self._dapi.readRegs(*uregs)
+            
         return regs
         
 
@@ -265,14 +297,15 @@ class BaseDBoard(object):
         :param int passwd: *password* to connect to the board
         
         '''
-        if self.dapi.regs.scsr.bits.access.value != DApiAccessLevel.NO.value: #@UndefinedVariable
-            self.disconnect()
-        self.dapi.cmd.connect(level,passwd) 
+        if self._dapi.regs.scsr.bits.access.value != DApiAccessLevel.NO.value: #@UndefinedVariable
+            self._dapi.cmd.disconnect()
+        if level != DApiAccessLevel.NO:
+            self._dapi.cmd.connect(level,passwd) 
         try:
-            self.dapi.regs.scsr.bits.access.set(level.value)
+            self._dapi.regs.scsr.bits.access.set(level.value)
         except DRegUndefinedError:
-            self.dapi.readReg(self.dapi.regs.scsr)
-            self.dapi.regs.scsr.bits.access.set(level.value)
+            self._dapi.readReg(self._dapi.regs.scsr)
+            self._dapi.regs.scsr.bits.access.set(level.value)
         
         
     def setWorkspace(self, workspace):
@@ -288,12 +321,12 @@ class BaseDBoard(object):
             raise ValueError('Undefined workspace!')
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
             if workspace.standby:
-                self.dapi.cmd.standby()
+                self._dapi.cmd.standby()
             else:
-                self.dapi.cmd.peripheralActivate(workspace.par)
+                self._dapi.cmd.peripheralActivate(workspace.par)
             self.regs.par.value = workspace.par
         else:
-            self.setReg(self.dapi.regs.par, workspace.par)
+            self.setReg(self._dapi.regs.par, workspace.par)
          
         #self._workspace = workspace
             
@@ -303,7 +336,7 @@ class BaseDBoard(object):
         :param bool refresh: specifies whether to update the in-memory register with the one on the board.
         ''' 
         if refresh:
-            self.dapi.readReg(self.regs.par)
+            self._dapi.readReg(self.regs.par)
         if self._workspace is None:
             self.onPARChanged(self.regs.par, self.regs.par.value, self.regs.par.value)
         return self._workspace
@@ -342,33 +375,33 @@ class BaseDBoard(object):
         ''' 
         
         if refresh:
-            self.dapi.readReg(self.regs.wer)
+            self._dapi.readReg(self.regs.wer)
         return self.regs.wer.value
         
     def getMicrocontrollerID(self):
         
-        ret = bytearray(self.dapi.cmd.getMcId(0))
-        ret.extend(self.dapi.cmd.getMcId(1))
+        ret = bytearray(self._dapi.cmd.getMcId(0))
+        ret.extend(self._dapi.cmd.getMcId(1))
         return ret
     
     def getLastReset(self, refresh=False):
         if refresh:
-            self.dapi.readReg(self.regs.scsr)
+            self._dapi.readReg(self.regs.scsr)
         return LastReset(self.regs.scsr.bits.reset.value)
 
     def getWatchdog(self, refresh=False):
         if refresh:
-            self.dapi.readReg(self.regs.scsr)
+            self._dapi.readReg(self.regs.scsr)
         return self.regs.scsr.bits.watchdog.value != 0
 
     
     def getSystemModeConfiguration(self, refresh=False):
         if refresh:
-            self.dapi.readReg(self.regs.smr)
+            self._dapi.readReg(self.regs.smr)
         try:
             return SystemModeConfig(self.regs.smr.value)
         except DRegUndefinedError:
-            self.dapi.readReg(self.regs.smr)
+            self._dapi.readReg(self.regs.smr)
             return SystemModeConfig(self.regs.smr.value)
         
     def setSystemModeConfiguration(self, flags):
@@ -376,33 +409,33 @@ class BaseDBoard(object):
     
     def getAccessLevel(self, refresh=False):
         if refresh or self.regs.scsr.isUndefined():
-            self.dapi.readReg(self.regs.scsr)
+            self._dapi.readReg(self.regs.scsr)
         return DApiAccessLevel(self.regs.scsr.bits.access.value)
     
     def getAdditionalBoard(self, refresh=False):
         if refresh or self.regs.scsr.isUndefined():
-            self.dapi.readReg(self.regs.scsr)
+            self._dapi.readReg(self.regs.scsr)
         return DApiAdditionalBoard(self.regs.scsr.bits.ab.value)
     
     def getPowerSupply(self, refresh=False):
         if refresh:
-            self.dapi.readReg(self.regs.psvr)
+            self._dapi.readReg(self.regs.psvr)
         return self.regs.psvr.value / 1000
     
     def getAnalogInput(self, index, refresh=False):
         reg = self.regs('an{0:d}r'.format(index))
         if refresh: 
-            self.dapi.readReg(reg)
+            self._dapi.readReg(reg)
         return reg.value
         
     def getSpeedReference(self, refresh=False):
         if refresh: 
-            self.dapi.readReg(self.regs.ref)
+            self._dapi.readReg(self.regs.ref)
         return self.regs.ref.value        
 
     def getPressure(self, refresh=False):
         if refresh: 
-            self.dapi.readReg(self.regs.prcr)
+            self._dapi.readReg(self.regs.prcr)
         return self.regs.prcr.value        
             
     def getFirmwareVersion(self):
@@ -429,11 +462,17 @@ class BaseDBoard(object):
      
     def disconnect(self):
         '''Disconnect the *MASTER* from the board.'''
-        self.dapi.cmd.disconnect()
+        self._dapi.cmd.disconnect()
+        try:
+            self._dapi.regs.scsr.bits.access.set(DApiAccessLevel.NO.value)  # @UndefinedVariable
+        except DRegUndefinedError:
+            self._dapi.readReg(self._dapi.regs.scsr)
+            self._dapi.regs.scsr.bits.access.set(DApiAccessLevel.NO.value)  # @UndefinedVariable
+        
         
     def reboot(self):
         '''Reboots the board.'''
-        self.dapi.cmd.reboot()
+        self._dapi.cmd.reboot()
         
     def motorStart(self, speed=None ):
         '''Starts the motor and optionally change the speed set point.
@@ -441,34 +480,34 @@ class BaseDBoard(object):
         :param int speed: The new speed set point. Default no change. 
         ''' 
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.motorStart(speed if speed is not None else 0)
+            self._dapi.cmd.motorStart(speed if speed is not None else 0)
         else: 
             if speed is not None:
-                self.setReg(self.dapi.regs.scr, speed, synchronous=False)
-            self.setBit(self.dapi.regs.smr.start)
+                self.setReg(self._dapi.regs.scr, speed, synchronous=False)
+            self.setBit(self._dapi.regs.smr.start)
             
     def motorStop(self):
         '''Stops the motor.''' 
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.motorStop()
+            self._dapi.cmd.motorStop()
         else: 
-            self.clearBit(self.dapi.regs.smr.start)
+            self.clearBit(self._dapi.regs.smr.start)
         
     def motorReverse(self, rev=None):
         '''Reverses the direction of rotation of the motor.'''
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.motorReverse()
+            self._dapi.cmd.motorReverse()
         else: 
-            self.toggleBit(self.dapi.regs.smr.reverse)
+            self.toggleBit(self._dapi.regs.smr.reverse)
              
             
     def motorForward(self):
         '''Sets forward direction (clockwise).'''
-        self.clearBit(self.dapi.regs.smr.reverse)
+        self.clearBit(self._dapi.regs.smr.reverse)
 
     def motorBackward(self):
         '''Sets backward direction (counter clockwise).'''
-        self.setBit(self.dapi.regs.smr.reverse)
+        self.setBit(self._dapi.regs.smr.reverse)
         
     def memoryStore(self, num):
         '''Stores the current set points into memory..
@@ -476,7 +515,7 @@ class BaseDBoard(object):
         :param int num: The memory slot number.
         '''
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.memoryStore(num)
+            self._dapi.cmd.memoryStore(num)
     
     def memoryRecall(self, num):
         '''Recalls the set points from memory..
@@ -484,7 +523,7 @@ class BaseDBoard(object):
         :param int num: The memory slot number.
         '''
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.memoryRecall(num)
+            self._dapi.cmd.memoryRecall(num)
         #self.getRegisters('setpoints')    
                
     def setMotorSpeed(self, speed):
@@ -492,7 +531,7 @@ class BaseDBoard(object):
         
         :param int speed: The new motor speed [rpm].
         '''
-        self.setReg(self.dapi.regs.scr, speed)
+        self.setReg(self._dapi.regs.scr, speed)
         
     def motorIncSpeed(self, inc):
         '''Increments the motor speed.
@@ -500,7 +539,7 @@ class BaseDBoard(object):
         :param int inc: The increment value.
         '''
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.motorIncSpeed(inc)
+            self._dapi.cmd.motorIncSpeed(inc)
         else:
             if self.motorSpeed()+inc > self.speed_range.upper:
                 inc = self.speed_range.upper - self.motorSpeed()
@@ -512,7 +551,7 @@ class BaseDBoard(object):
         :param int inc: The decrement value.
         '''
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.motorDecSpeed(inc)
+            self._dapi.cmd.motorDecSpeed(inc)
         else:
             if self.motorSpeed()-inc < self.speed_range.lower:
                 inc = self.motorSpeed() 
@@ -522,42 +561,42 @@ class BaseDBoard(object):
     def motorSpeed(self):
         '''Returns the motor speed set point'''
         try:
-            return self.dapi.regs.scr.value
+            return self._dapi.regs.scr.value
         except DRegUndefinedError:
             self.getRegisters('scr')
-            return self.dapi.regs.scr.value
+            return self._dapi.regs.scr.value
 
         
     def motorRealSpeed(self):
         '''Returns the real motor speed'''
         try:
-            return self.dapi.regs.msr.value
+            return self._dapi.regs.msr.value
         except DRegUndefinedError:
             self.getRegisters('msr')
-            return self.dapi.regs.msr.value
+            return self._dapi.regs.msr.value
         
     def setMotorCurrent(self, torque):
         '''Sets the maximum motor current
         
         :param int torque: The new maximum motor current [mA].
         '''
-        self.setReg(self.dapi.regs.ccr, torque) 
+        self.setReg(self._dapi.regs.ccr, torque) 
         
     def motorCurrent(self):
         '''Returns the maximum motor current set point'''
         try:
-            return self.dapi.regs.ccr.valued
+            return self._dapi.regs.ccr.valued
         except DRegUndefinedError:
             self.getRegisters('ccr')
-            return self.dapi.regs.ccr.value
+            return self._dapi.regs.ccr.value
         
     def motorRealCurrent(self):
         '''Returns the real motor current'''
         try:
-            return self.dapi.regs.a256dcr.value
+            return self._dapi.regs.a256dcr.value
         except DRegUndefinedError:
             self.getRegisters('a256dcr')
-            return self.dapi.regs.a256dcr.value
+            return self._dapi.regs.a256dcr.value
     
     def setGearRatio(self, numerator, denominator):
         '''Sets the hand piece gear ratio.
@@ -570,26 +609,26 @@ class BaseDBoard(object):
     def isLightEnabled(self):
         '''Checks if light is enabled.'''
         try:
-            return self.dapi.regs.smr.bits.light.value
+            return self._dapi.regs.smr.bits.light.value
         except DRegUndefinedError:
             self.getRegisters('smr')
-            return self.dapi.regs.smr.bits.light.value
+            return self._dapi.regs.smr.bits.light.value
 
     def isLightAlternate(self):
         '''Checks if the alternate light is activated.'''
         try:
-            return self.dapi.regs.alr.value != 0
+            return self._dapi.regs.alr.value != 0
         except DRegUndefinedError:
             self.getRegisters('alr')
-            return self.dapi.regs.alr.value != 0
+            return self._dapi.regs.alr.value != 0
         
     def isMotorRunning(self):
         '''Checks if motor is running.'''
         try:
-            return self.dapi.regs.ssr1.bits.rotation.value
+            return self._dapi.regs.ssr1.bits.rotation.value
         except DRegUndefinedError:
             self.getRegisters('ssr1')
-            return self.dapi.regs.ssr1.bits.rotation.value
+            return self._dapi.regs.ssr1.bits.rotation.value
     
     def isMotorStarted(self):
         '''Checks if motor is started.'''
@@ -603,10 +642,10 @@ class BaseDBoard(object):
     def isMotorReverse(self):
         '''Checks if CCW direction is selected.'''
         try:
-            return self.dapi.regs.smr.bits.reverse.value
+            return self._dapi.regs.smr.bits.reverse.value
         except DRegUndefinedError:
             self.getRegisters('smr')
-            return self.dapi.regs.smr.bits.reverse.value
+            return self._dapi.regs.smr.bits.reverse.value
 
     
     def lightOn(self, on=True):
@@ -615,23 +654,23 @@ class BaseDBoard(object):
             self.lightOff()
         else: 
             if self.dmode == DBoardPreferredDapiMode.COMMAND:
-                self.dapi.cmd.lightOn()
+                self._dapi.cmd.lightOn()
             else: 
-                self.setBit(self.dapi.regs.smr.bits.light)
+                self.setBit(self._dapi.regs.smr.bits.light)
 
     def lightOff(self):
         '''Disables the light.'''
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.lightOff()
+            self._dapi.cmd.lightOff()
         else: 
-            self.clearBit(self.dapi.regs.smr.bits.light)
+            self.clearBit(self._dapi.regs.smr.bits.light)
             
     def lightAuto(self, auto):
         '''Sets the light mode.
         
         :param bool auto: True, sets the automatic mode, False, sets the direct (manual) mode. 
         '''
-        self.setBit(self.dapi.regs.smr.bits.lightauto, int(auto))
+        self.setBit(self._dapi.regs.smr.bits.lightauto, int(auto))
     
 
     def lightIntensity(self, intensity):
@@ -639,9 +678,9 @@ class BaseDBoard(object):
         
         :param int intensity: The light intensity in mA'''  
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.lightIntensity(intensity)
+            self._dapi.cmd.lightIntensity(intensity)
         else: 
-            self.setReg(self.dapi.regs.lir, intensity)
+            self.setReg(self._dapi.regs.lir, intensity)
 
     def lightAlternate(self, alternate=True):
         '''chosen between normal (white) or alternative (UV) light
@@ -649,9 +688,9 @@ class BaseDBoard(object):
         :param bool alternate: True = Alternate ; False = Normal
         ''' 
         #if self.dmode == DBoardPreferredDapiMode.COMMAND:
-        #    self.dapi.cmd.lightAlternate(0x100 * int(alternate))
+        #    self._dapi.cmd.lightAlternate(0x100 * int(alternate))
         #else: 
-        self.setReg(self.dapi.regs.alr, 0x100 * int(alternate))
+        self.setReg(self._dapi.regs.alr, 0x100 * int(alternate))
 
             
     def calibrate(self, part, phase):
@@ -660,7 +699,7 @@ class BaseDBoard(object):
         :param int part: Index of part (sensor) to calibrate.
         :param int phase: Pahse number of calibration.   
         '''
-        self.dapi.cmd.calibrate(part, phase)
+        self._dapi.cmd.calibrate(part, phase)
         
     def setFactoryData(self, sn, fd, hv=None):
         '''Sets the factory data.
@@ -676,7 +715,7 @@ class BaseDBoard(object):
         self.log.info('Set factory data. SN:{0:04d}, FD:{1:s}, HV:{2:d}.{3:02d}'.format(sn, fd.isoformat(), hv[0], hv[1]) )
         
         if self.dmode == DBoardPreferredDapiMode.COMMAND:
-            self.dapi.cmd.factSetSysinfo(sn, fd, hv)
+            self._dapi.cmd.factSetSysinfo(sn, fd, hv)
         else: 
             self.setRegisters(snr=sn, fdr=dateToWord(fd), hvr = versionToWord(*hv), synchronous=True)        
                
@@ -691,45 +730,45 @@ class BaseDBoard(object):
         return (self.regs.snr.value, wordToDate(self.regs.fdr.value), wordToVersion(self.regs.hvr.value))
         
         
-    def refreshAll(self):
-        self.refreshState()
-        self.refreshSetpoints()
-        self.refreshDebug()
-        self.refreshSystem()
+    def refreshAll(self, undefined_only=False):
+        self.refreshState(undefined_only=undefined_only)
+        self.refreshSetpoints(undefined_only=undefined_only)
+        self.refreshDebug(undefined_only=undefined_only)
+        self.refreshSystem(undefined_only=undefined_only)
         
-    def refreshState(self, part=None):
+    def refreshState(self, part=None, undefined_only=False):
         #self.log.debug('refreshState')
-        self.getRegisters('header', refresh=True)
+        self.getRegisters('header', refresh=True, undefined_only=undefined_only)
         if part is None:
-            self.getRegisters('state', refresh=True)
+            self.getRegisters('state', refresh=True, undefined_only=undefined_only)
         else:
-            self.getRegisters('ssr1', 'ssr2', 'msr', 'a256dcr', refresh=True)
+            self.getRegisters('ssr1', 'ssr2', 'msr', 'a256dcr', refresh=True, undefined_only=undefined_only)
             if self.regs.ssr1.wanr_err != 0 and part != 8:
-                self.getRegisters('wer', refresh=True)
-            addr = self.regs.state.addr+(part*4)
-            if part > 0: 
-                self.getRegisters(self.regs[addr:addr+4], refresh=True)
+                self.getRegisters('wer', refresh=True, undefined_only=undefined_only)
+            if part > 0:
+                addr = self.regs.state.addr+(part*4) 
+                self.getRegisters(self.regs[addr:addr+4], refresh=True, undefined_only=undefined_only)
                 
 
-    def refreshSetpoints(self):
+    def refreshSetpoints(self, undefined_only=False):
         #self.log.debug('refreshSetpoints')
-        self.getRegisters('setpoints', refresh=True)
+        self.getRegisters('setpoints', refresh=True, undefined_only=undefined_only)
         
-    def refreshDebug(self):
+    def refreshDebug(self, undefined_only=False):
         #self.log.debug('refreshDebug')
-        self.getRegisters('debug', refresh=True)
+        self.getRegisters('debug', refresh=True, undefined_only=undefined_only)
 
-    def refreshAnalogInputs(self):
+    def refreshAnalogInputs(self, undefined_only=False):
         #self.log.debug('refreshDebug')
-        self.getRegisters('prcr', 'elcr', 'an0r', 'an1r', 'psvr', 'esvr',refresh=True)
+        self.getRegisters('prcr', 'elcr', 'an0r', 'an1r', 'psvr', 'esvr',refresh=True, undefined_only=undefined_only)
 
-    def refreshSystem(self):
+    def refreshSystem(self, undefined_only=False):
         #self.log.debug('refreshSystem')
-        self.getRegisters('system', refresh=True)
+        self.getRegisters('system', refresh=True, undefined_only=undefined_only)
 
         
     def eepromReset(self):
-        self.dapi.cmd.factEerpomReset()
+        self._dapi.cmd.factEerpomReset()
         
     def flashFirm(self, firm, callback=None):
         
@@ -740,28 +779,40 @@ class BaseDBoard(object):
             else:
                 return ord(c)
         self.log.info('Firmware programming has started....')
-        with open( firm.filename, 'rb' ) as f:
+        with open( firm, 'rb' ) as f:
             f.seek(0, os.SEEK_END)
             size = f.tell()
-            self.dapi.cmd.flashBegin(size)
+            self._dapi.cmd.flashBegin(size)
             f.seek(0)
             i = 0
             
             while i < size:
-                self.dapi.cmd.flashData([_readByte(x) for x in range(CommandMessage.MAX_DATA_SIZE)])
+                self._dapi.cmd.flashData([_readByte(x) for x in range(CommandMessage.MAX_DATA_SIZE)])
                 i+=CommandMessage.MAX_DATA_SIZE
                 self.log.debug('Programming block {0} / {1}'.format(i//CommandMessage.MAX_DATA_SIZE,size//CommandMessage.MAX_DATA_SIZE))
                 if callback is not None:
                     callback(i, size)
                     
 
-        self.dapi.cmd.flashEnd()
+        self._dapi.cmd.flashEnd()
         self.log.info('Firmware programming is complete!')
         
-        
+    def onDComConnectionChanged(self, connected):
+        self.log.debug(f'onDComConnectionChanged : {connected}')
+        self._connection_changed.emit(connected, self.getAccessLevel())
+            
     def onPARChanged(self, reg, old, value):
         self.log.debug('onPARChanged 0x{0:04x} => 0x{1:04x}'.format(old,value))
         self._workspace = self.workspaces.getByPAR(value)
+        self.log.info(f'{self._workspace!s} is activated.'.format(old,value))
+        self.getRegisters('ctr', refresh=True)
+        self._workspace_changed.emit(self._workspace)
+        
+    def onSCSRChanged(self, reg, old, value):
+        self.log.debug('onSCSRChanged 0x{0:04x} => 0x{1:04x}'.format(old,value))
+        
+        if (old & DAPI_ACCESS_LEVEL_MASK) != (value & DAPI_ACCESS_LEVEL_MASK):
+            self._connection_changed.emit(self._dapi.dcom.isOpen(), self.getAccessLevel())
         
     def setDacSignal(self, dac, signal):
         '''Sets the signal that the DAC should generate.
@@ -771,16 +822,25 @@ class BaseDBoard(object):
         '''
         assert isinstance(dac, DacChannel) 
         assert isinstance(signal, DacSignal)
-        self.setReg(self.dapi.regs.dacsr[dac.value], signal.value, synchronous=True)        
-        
+        self.setReg(self._dapi.regs.dacsr[dac.value], signal.value, synchronous=True)        
         
+    
+    @property
+    def dapi(self):
+        return self._dapi
+    @property
+    def dcom(self):
+        return self._dapi.dcom
     @property
     def regs(self):
-        return self.dapi.regs
+        return self._dapi.regs
     @property
     def workspaces(self):
         return self._workspaces
     @property
+    def workspace(self):
+        return self._workspace
+    @property
     def name(self):
         return self.getName().lower()
     @property
@@ -789,6 +849,12 @@ class BaseDBoard(object):
     @property
     def analogInputs(self):
         return self._analog_inputs.values()
+    @property
+    def workspaceChanged(self):
+        return self._workspace_changed    
+    @property
+    def connectionChanged(self):
+        return self._connection_changed    
     
     
     
@@ -811,11 +877,11 @@ class DBoard(BaseDBoard):
         BaseDBoard.__init__(self, dapi, dmode=dmode)
         self._analog_inputs = dict( 
                 ((sensor.name, sensor) for sensor in (
-                    PressureSensor(self, 'pressure', self.regs.prcr, cal_state_index=1),
-                    ReferenceSensor(self, 'electric', self.regs.elcr),
-                    ElectricalSensor(self, 'analog_0', self.regs.an0r),
-                    ElectricalSensor(self, 'analog_1', self.regs.an1r), 
-                    PowerSupply(self, 'power', self.regs.psvr),)
+                    PressureSensor(self),
+                    ReferenceSensor(self),
+                    Analog0Sensor(self),
+                    Analog1Sensor(self), 
+                    PowerSupply(self),)
                     )
             )
         self._debug_values = ( DebugValue(self, i) for i in range(self.regs.dmr.size) )
