@@ -16,7 +16,12 @@ class DacSignalMb60(DacSignal):
     
 
 class Board60(DBoard):
-    '''Class for MB-60 Dassym's board.'''
+    '''Class for MB-60 Dassym's board.
+    
+    .. inheritance-diagram:: Board60
+        :parts: 1
+    
+    '''
     
     number = '60'
     '''Board type number (str)'''
