@@ -32,9 +32,12 @@ class Memory(object):
         
             
     def isCurrent(self):
-        if self._values[0] & self.SMR_MASK != self.board.regs.setpoints.values[0] & self.SMR_MASK:
-            return False  
-        return self._values[1:] == self.board.regs.setpoints.values[1:]
+        try:
+            if self._values[0] & self.SMR_MASK != self.board.regs.setpoints.values[0] & self.SMR_MASK:
+                return False  
+            return self._values[1:] == self.board.regs.setpoints.values[1:]
+        except IndexError:
+            return False
         
     def isDefined(self):
         return len(self._values)==self.SIZE 
