@@ -7,6 +7,7 @@ Created on 8 févr. 2021
 from .base import BaseRegElement, DRegException, DRegAccess, DRegChangedSignal
 from .. import dreg
 from ..common import REG_MAX_VALUE  
+
  
 
 class DRegUndefinedError(DRegException):
@@ -39,7 +40,8 @@ class Register(BaseRegElement):
     
     
     def __init__(self, name, parent=None, addr=None, size=1, descriptions=None, shortname=None, min=0, max=REG_MAX_VALUE, access=dreg.DRegAccess.READ): #@ReservedAssignment
-        assert parent is None or isinstance(parent, (dreg.RegContainer, dreg.RegGroup, dreg.RegisterArray )), 'parent is '+type(parent).__name__
+        #assert parent is None or isinstance(parent, (dreg.RegContainer, dreg.RegGroup, dreg.RegisterArray )), 'parent is '+type(parent).__name__
+        assert isinstance(parent, (dreg.RegContainer, dreg.RegGroup, dreg.RegisterArray )), 'parent is '+type(parent).__name__
         super().__init__(name, parent, addr, size, descriptions, shortname)
         self._access = access
         self.min = min
@@ -84,8 +86,9 @@ class Register(BaseRegElement):
         self._modified = modified and old != value 
         if self._undefined or old != value:
             self.container.values[self.addr] = value
+            self._undefined = False
             self._valueChanged(old)
-        self._undefined = False
+        
         return value
             
     def set(self, value):
@@ -100,6 +103,10 @@ class Register(BaseRegElement):
     def clear(self):
         return self.set(0)
     
+    def reset(self):
+        self._modified = False 
+        self._undefined = True
+    
     def get(self):
         if self._undefined:
             raise DRegUndefinedError(self, 'get', 'Undefined value in register '+self.name)
@@ -115,7 +122,6 @@ class Register(BaseRegElement):
         return self._changed
         
 class ReservedRegister(Register):
-    
     pass
 
 class RegisterBits(object):
